﻿Imports System
Imports System.Configuration
Imports System.Globalization
Imports System.IO
Imports System.Web
Imports System.Xml

Namespace CustomLocalization.Web
    Public Class XapHandler
        Implements IHttpHandler
        Private Culture As CultureInfo
        Private Context As HttpContext
        Private ResourceSet As String

        Public ReadOnly Property IsReusable() As Boolean Implements IHttpHandler.IsReusable
            Get
                Return False
            End Get
        End Property

        Public Const BASERESOURCEPATH As String = "\Resources\"

        Private Shared _resourceBasePath As String = Nothing

        Public Shared ReadOnly Property ResourceBasePath() As String
            Get
                If _resourceBasePath Is Nothing Then
                    _resourceBasePath = ConfigurationManager.AppSettings("ResourceBasePath")
                    If String.IsNullOrEmpty(_resourceBasePath) Then
                        _resourceBasePath = BASERESOURCEPATH
                    End If

                    If _resourceBasePath IsNot Nothing AndAlso (Not _resourceBasePath.EndsWith("\")) Then
                        _resourceBasePath &= "\"
                    End If
                End If

                Return _resourceBasePath
            End Get
        End Property

        Private Shared resourceCacheDurationSet As Boolean = False

        Private Shared _resourceCacheDuration As TimeSpan

        Public Shared ReadOnly Property ResourceCacheDuration() As TimeSpan
            Get
                If Not resourceCacheDurationSet Then
                    resourceCacheDurationSet = True

                    ' Set to 12 hours
                    _resourceCacheDuration = New TimeSpan(12, 0, 0)

                    Dim strDuration As String = ConfigurationManager.AppSettings("ResourceCacheDuration")
                    If Not String.IsNullOrEmpty(strDuration) Then
                        Dim duration As Integer
                        If Integer.TryParse(strDuration, duration) Then
                            _resourceCacheDuration = New TimeSpan(duration, 0, 0)
                        End If
                    End If
                End If

                Return _resourceCacheDuration
            End Get
        End Property

        Public Sub ProcessRequest(ByVal context As HttpContext) Implements IHttpHandler.ProcessRequest
            Me.Context = context
            Dim culture As String = context.Request.Params("c")


            ResourceSet = context.Request.Params("rs")

            If Not String.IsNullOrEmpty(Culture) Then
                Try
                    Me.Culture = New CultureInfo(Culture)
                Catch ex As Exception
                    Dim errMsg As String = ex.Message

                    ' throw an error
                    Utilities.SendResponse(Me.Context, 500, "The string " & Culture & " is not recognized as a valid culture.")
                    Return
                End Try
            End If

            Dim xapWithoutResources As String = Me.Context.Server.MapPath(Me.Context.Request.Path)

            If String.IsNullOrEmpty(xapWithoutResources) OrElse (Not File.Exists(xapWithoutResources)) Then
                Utilities.SendResponse(Me.Context, 500, "The XAP file does not exist.")
                Return
            Else
                Dim localizedXapFilePath As String = CreateLocalizedXapFile(xapWithoutResources)

                Utilities.TransmitFile(context, localizedXapFilePath, "application/x-silverlight-app", False)
            End If
        End Sub

        Private Function CreateLocalizedXapFile(ByVal filePath As String) As String
            Dim plainXap As New FileInfo(filePath)
            Dim localizedXapFilePath As String = plainXap.FullName

            Try
                ' Get the localized XAP file
                Dim localizedXap As New FileInfo(plainXap.DirectoryName & "\" & Culture.Name & "\" & plainXap.Name)

                ' Get the RESX file for the locale
                Dim resxFile As New FileInfo(GetResourceFilePath(Context, ResourceSet, Culture.Name))

                ' Check to see if the file already exists and is up to date
                If (Not localizedXap.Exists) OrElse (localizedXap.LastWriteTime < plainXap.LastWriteTime) OrElse (localizedXap.LastWriteTime < resxFile.LastWriteTime) Then
                    If Not Directory.Exists(localizedXap.DirectoryName) Then
                        Directory.CreateDirectory(localizedXap.DirectoryName)
                    End If

                    ' Copy the XAP without resources
                    localizedXap = plainXap.CopyTo(localizedXap.FullName, True)

                    ' Inject the resources into the plain XAP, turning it into a localized XAP
                    If Not InjectResourceIntoXAP(localizedXap, resxFile) Then
                        localizedXap.Delete()
                    End If
                End If

                If File.Exists(localizedXap.FullName) Then
                    localizedXapFilePath = localizedXap.FullName
                End If
            Catch ex As Exception
                ' If any error occurs, throw back the error message
                If Not File.Exists(localizedXapFilePath) Then
                    Utilities.SendResponse(Context, 500, ex.Message)
                End If
            End Try

            Return localizedXapFilePath
        End Function

        Private Function InjectResourceIntoXAP(ByVal localizedXapFile As FileInfo, ByVal localizedResxFile As FileInfo) As Boolean
            If localizedXapFile.Exists AndAlso localizedResxFile.Exists Then
                Try
                    Utilities.AddFileToZip(localizedXapFile.FullName, localizedResxFile.FullName, String.Empty)
                    Return True
                Catch
                    Return False
                End Try
            End If

            Return False
        End Function

        Public Shared Function GetResourceFilePath(ByVal context As HttpContext, ByVal resourceSet As String, ByVal culture As String) As String
            If context Is Nothing Then
                Return Nothing
            End If
            If String.IsNullOrEmpty(culture) Then
                Return Nothing
            End If

            Dim resxFilePath As String = resourceSet & "." & culture & ".resx"

            Dim folderPath As String = context.Server.MapPath(ResourceBasePath)

            Dim resxFile As New FileInfo(folderPath & resxFilePath)

            Dim cacheDuration As TimeSpan = ResourceCacheDuration

            If (Not resxFile.Exists) OrElse (resxFile.LastWriteTime.Add(cacheDuration) < Date.Now) Then
                Try
                    If Not Directory.Exists(folderPath) Then
                        Directory.CreateDirectory(folderPath)
                    End If

                    Dim xmlDoc As XmlDocument = DbResXConverter.GenerateResXFileNormalizedForCulture(resourceSet, culture)

                    xmlDoc.Save(resxFile.FullName)
                Catch ex As Exception
                    If Not resxFile.Exists Then
                        Utilities.SendResponse(context, 500, ex.Message)
                    End If
                End Try
            End If

            Return resxFile.FullName
        End Function
    End Class
End Namespace